@@ -23,9 +23,9 @@
 
 
 '''
-VERSION = u'2.2.1'
+VERSION = u'2.2.3-1'
 
 from redssh.redssh import RedSSH
-from redssh.redssh import exceptions
+from redssh import exceptions
 import redssh.libssh2 as libssh2
 
